﻿Imports System.ComponentModel
Imports System.Text
Imports IdeaBlade.EntityModel

' This sample uses Employee and Territory from the NorthwindIB database and shows two ways to work with
' many-to-many mappings with payload.
'
' In a many-to-many mapping with payload the mapping table is allowed to hold other information. The mapping table is mapped to 
' an Entity and managed like any other Entity in the system. This sample uses the EmployeeTerritory 
' table. To create a new mapping, you explicitly create a new EmployeeTerritory and assign the Employee 
' and Territory to it like employeeTerritory.Employee = employee and employeeTerritory.Territory = territory.
'
'   This sample does not save to the database, but you can do so easily by caling 
'   _entityManager.SaveChangesAsync();

Namespace SilverlightConsole
  Public Class CodeSamples
	Public Sub New()
	End Sub

	Public Sub Start()
      Coroutine.Start(Function() Samples())
	End Sub

    Private Function Samples() As IEnumerable(Of Func(Of INotifyCompleted))

      ' Because Silverlight is asynchronous, we use the coroutine syntax to call these methods in order
      ' Note that coroutines are not a part of nor are necessary for performing operations with entities in a many-to-many
      ' relationship. 
      ' If you're not familiar with coroutines, please disregard the "yield return's" and focus on the logic of the methods below.

      Dim operationList = New List(Of Func(Of INotifyCompleted))

      ' The next three samples use the EmployeeTerritoryNoPayload mapping table

      'INSTANT VB TODO TASK: VB does not support iterators and has no equivalent to the C# 'yield' keyword:
      'yield Return EmployeeTerritoryNavigationNoPayload()
      Dim f1 As Func(Of INotifyCompleted) =
       Function()
         SLConsole.WriteLine("Note: The server may take a few seconds to initialize." & vbLf)
         Return EmployeeTerritoryNavigationWithPayload()
       End Function
      operationList.Add(f1)

      'INSTANT VB TODO TASK: VB does not support iterators and has no equivalent to the C# 'yield' keyword:
      'yield Return AddTerritoryWithPayload()
      Dim f2 As Func(Of INotifyCompleted) =
       Function()
         Return AddTerritoryWithPayload()
       End Function
      operationList.Add(f2)

      'INSTANT VB TODO TASK: VB does not support iterators and has no equivalent to the C# 'yield' keyword:
      'yield Return DeleteTerritoryWithPayload()
      Dim f3 As Func(Of INotifyCompleted) =
      Function()
        Return DeleteTerritoryWithPayload()
      End Function
      operationList.Add(f3)

      'INSTANT VB TODO TASK: VB does not support iterators and has no equivalent to the C# 'yield' keyword:
      'yield Return Coroutine.Return(Nothing)
      Dim f4 As Func(Of INotifyCompleted) =
     Function()
       Return Coroutine.Return(Nothing)
     End Function
      operationList.Add(f4)

      Return operationList
    End Function

	' The three sample methods below are where the "many-to-many" operations take place.
	Private Function EmployeeTerritoryNavigationWithPayload() As INotifyCompleted
	  SLConsole.WriteLine("EmployeeTerritoryNavigationWithPayload Started...")
	  _entityManager.Clear()

	  ' Get an Employee and include their Territories so we can iterate over them below
	  Dim queryOp = _entityManager.Employees.Include("EmployeeTerritories.Territory").Where(Function(e) e.EmployeeID = 1).AsScalarAsync().First()

	  AddHandler queryOp.Completed, Sub(s, e)
		Dim employee = queryOp.Result
		' employee.employeeTerritories is available immediately because it was cached by .Include("EmployeeTerritories.Territories")
		Dim employeeTerritories = employee.EmployeeTerritories.ToList()

		SLConsole.WriteLine("Employee " & employee.FirstName + employee.LastName & " is assigned to " & employee.EmployeeTerritories.Count().ToString() & " territories:")

		' employeeTerritory.Territory is available immediately because it was cached by .Include("EmployeeTerritories.Territories")
		For Each employeeTerritory In employeeTerritories
		  SLConsole.WriteLine(employeeTerritory.Territory.TerritoryDescription.Trim())
		Next employeeTerritory
		SLConsole.WriteLine()
	  End Sub
	  Return queryOp
	End Function

	Private Function AddTerritoryWithPayload() As INotifyCompleted
	  Dim territoryToBeAdded As Territory
	  SLConsole.WriteLine("AddTerritoryWithPayload Started...")
	  _entityManager.Clear()

	  ' Get an Employee and include their Territories
	  Dim queryOp = _entityManager.Employees.Include("EmployeeTerritories.Territory").Where(Function(e) e.EmployeeID = 1).AsScalarAsync().First()

	  AddHandler queryOp.Completed, Sub(s, e)
		Dim employee = queryOp.Result

		SLConsole.WriteLine("Employee " & employee.FirstName & " " & employee.LastName & " is currently assigned to " & employee.EmployeeTerritories.Count().ToString() & " territories.")

		' Create a new Territory
		territoryToBeAdded = New Territory()
		territoryToBeAdded.TerritoryDescription = "Oakland"
		territoryToBeAdded.RegionID = 2
		' Remember to add it to the EntityManager
		_entityManager.AddEntity(territoryToBeAdded)

		SLConsole.WriteLine("Assigning newTerritory [" & territoryToBeAdded.TerritoryDescription & "] to " & employee.FirstName & " " & employee.LastName)

		' Create a new EmployeeTerritory (the mapping entity/table)
		Dim newEmployeeTerritory As New EmployeeTerritory()
		newEmployeeTerritory.Employee = employee
		newEmployeeTerritory.Territory = territoryToBeAdded
		' Remember to add it to the EntityManager
		_entityManager.AddEntity(newEmployeeTerritory)

		SLConsole.WriteLine("Now " & employee.FirstName & " " & employee.LastName & " is currently assigned to " & employee.EmployeeTerritories.Count() & " territories.")
		SLConsole.WriteLine()
	  End Sub
	  Return queryOp
	End Function

	Private Function DeleteTerritoryWithPayload() As INotifyCompleted
	  SLConsole.WriteLine("DeleteTerritoryWithPayload Started...")
	  _entityManager.Clear()

	  ' Get an Employee and include their Territories
	  Dim queryOp = _entityManager.Employees.Include("EmployeeTerritories.Territory").Where(Function(e) e.EmployeeID = 1).AsScalarAsync().First()

	  AddHandler queryOp.Completed, Sub(s, e)
		Dim employee = queryOp.Result

		SLConsole.WriteLine("Employee " & employee.FirstName & " " & employee.LastName & "is currently assigned to " & employee.EmployeeTerritories.Count() & " territories.")

		' Get an EmployeeTerritory mapping that is to be deleted
		Dim employeeTerritoryToBeremoved As EmployeeTerritory = employee.EmployeeTerritories.First()

		SLConsole.WriteLine("Unassigning Territory [" & employeeTerritoryToBeremoved.Territory.TerritoryDescription.Trim() & "] from " & employee.FirstName & " " & employee.LastName)

		' Delete the EmployeeTerritory
		' Note that in this example, we did not delete any Territories, only the mapping between Employee and Territory
		employeeTerritoryToBeremoved.EntityAspect.Delete()

		SLConsole.WriteLine("Now " & employee.FirstName & " " & employee.LastName & " is currently assigned to " & employee.EmployeeTerritories.Count() & " territories.")
		SLConsole.WriteLine()
	  End Sub
	  Return queryOp
	End Function

	Private _entityManager As New NorthwindIBEntities()
  End Class
End Namespace
